-- Hook Chat Links
local Original_ChatEdit_InsertLink = ChatEdit_InsertLink
function ChatEdit_InsertLink(item, ...)
   local editbox = GFL_MainFrame_EditBox
   local focus = editbox:HasFocus()
   local visible = editbox:IsVisible()

   if item and focus and visible and IsShiftKeyDown() then
      editbox:Insert(item)
      return true
   end
   return Original_ChatEdit_InsertLink(item, ...)
end

-- Private Data
local ADDON_PREFIX = "GFL_Prefix"
local ADDON_NAME = "GuildFarmingList"

local Utils = GFL_UtilFuncs

local SecsPerMin = 60
local SecsPerHour = 60 * SecsPerMin
local SecsPerDay = 24 * SecsPerHour
local DaysToSave = 30

local MaxScrollItemsShown = nil --Number of Items the ScrollFrame can show at the same time
local MaxScrollItems = 75 -- Maximum number of items that can be listed

-- ScrollBar constants
local ScrollBarOffset = 0
local ScrollBarMin = 0
local ScrollBarMax = 800

-- Methods
local function ItemKnownListed(item)
   local known, index = nil, nil
   item = Utils.GetItemLink(item)
   local id, name = Utils.ParseLink(item)

   for i = 1, #GFL_ItemsNeeded do
      if id == Utils.ParseLink(GFL_ItemsNeeded[i].Item) then
         known, index = true, i
         break
      end
   end

   if not known then
      for i = 1, #GFL_ItemsNeeded do
         if name == GFL_ItemsNeeded[i].Item then
            known, index = true, i
            break
         end
      end
   end

   return known, index
end

local function ItemKnownDelisted(item)
   local known, index = nil, nil
   item = Utils.GetItemLink(item)
   local id, name = Utils.ParseLink(item)

   for i = 1, #GFL_ItemsNoLongerNeeded do
      if id == Utils.ParseLink(GFL_ItemsNoLongerNeeded[i].Item) then
         known, index = true, i
         break
      end
   end

   if not known then
      for i = 1, #GFL_ItemsNoLongerNeeded do
         if name == GFL_ItemsNoLongerNeeded[i].Item then
            known, index = true, i
            break
         end
      end
   end

   return known, index 
end

local function AddonSendMessage(remove, link, date)
   if not remove then
      SendAddonMessage(ADDON_PREFIX, "A=A;L="..link..";D="..date, "GUILD")
   else
      SendAddonMessage(ADDON_PREFIX, "A=R;L="..link..";D="..date, "GUILD")
   end
end

local function AddonSendMessageDirect(remove, link, date, player)
   if not remove then
      SendAddonMessage(ADDON_PREFIX, "A=A;L="..link..";D="..date, "WHISPER", player)
   else
      SendAddonMessage(ADDON_PREFIX, "A=R;L="..link..";D="..date, "WHISPER", player)
   end

end

local function ParseChat(msg)
   local action = strmatch(msg, ".*A=([^;]*).*")
   local item = strmatch(msg, ".*L=([^;]*).*")
   local date = strmatch(msg, ".*D=([^;]*).*")
   date = tonumber(date)
   return action, item, date
end

local function AddItem(item, date)
   if #GFL_ItemsNeeded < MaxScrollItems then
      local delisted, delistedIndex = ItemKnownDelisted(item)

      if (delisted and not date) or
         (delisted and GFL_ItemsNoLongerNeeded[delistedIndex].Date <= date) or
         (not delisted) then
         local listed, listedIndex = ItemKnownListed(item)
         if not listed then
            local t  = {}
            t.Item = item
            if date then
               t.Date = date
            else
               t.Date = GetServerTime()
            end
            table.insert(GFL_ItemsNeeded, 1, t)
            if delisted then
               table.remove(GFL_ItemsNoLongerNeeded, delistedIndex)
            end
            GFL_Main.ScrollFrame.OnUpdate(GFL_MainFrame_ScrollFrame)
            if not date then
               AddonSendMessage(false, t.Item, t.Date)
            end
         else
            if date then
               GFL_ItemsNeeded[listedIndex].Date = math.max(date, GFL_ItemsNeeded[listedIndex].Date)
            end
         end
      end
   end
end

local function RemoveItem(item, date)
   local listed, listedIndex = ItemKnownListed(item)

   if (listed and not date) or
      (listed and GFL_ItemsNeeded[listedIndex].Date < date) or
      (not listed) then
      if listedIndex then
         GFL_Main.RemoveItemByIndex(listedIndex, date)
      end
   end
end

-- Main Functions
GFL_Main = {}
GFL_Main.MainFrame = {}
GFL_Main.MiniMap = {}
GFL_Main.DraggingButton = {}
GFL_Main.EditBox = {}
GFL_Main.ScrollFrame = {}

GFL_ItemsNeeded = {}
GFL_ItemsNoLongerNeeded = {}
GFL_Options = {}
GFL_Options.Minimap = {}
GFL_Options.MainFrame = {}


function GFL_Main.MainFrame.OnEvent(self, event, ...)
   local args = {...}
	local currentPlayer = UnitName("player")
   

   if event == "CHAT_MSG_ADDON" and args[1] == ADDON_PREFIX then
      if args[2] then
         local action, item, date = ParseChat(args[2])
         if action == "R" and args[5] ~= currentPlayer then
            --Remove
            RemoveItem(item, date)
         elseif action == "A" and args[5] ~= currentPlayer then
            -- Add
            AddItem(item, date)
         elseif action == "S" then
            if args[5] == currentPlayer then
               for i = 1, #GFL_ItemsNeeded do
                  AddonSendMessage(false, GFL_ItemsNeeded[i].Item, GFL_ItemsNeeded[i].Date)
               end

               for i = 1, #GFL_ItemsNoLongerNeeded do
                  AddonSendMessage(true, GFL_ItemsNoLongerNeeded[i].Item, GFL_ItemsNoLongerNeeded[i].Date)
               end
            else
               for i = 1, #GFL_ItemsNeeded do
                  AddonSendMessageDirect(false, GFL_ItemsNeeded[i].Item, GFL_ItemsNeeded[i].Date, args[5])
               end

               for i = 1, #GFL_ItemsNoLongerNeeded do
                  AddonSendMessageDirect(true, GFL_ItemsNoLongerNeeded[i].Item, GFL_ItemsNoLongerNeeded[i].Date, args[5])
               end
            end
         elseif action == "T" then
         end
      end
   elseif event == "ADDON_LOADED" and args[1] == ADDON_NAME then
      for i = #GFL_ItemsNoLongerNeeded, 1, -1 do
         if tonumber(GFL_ItemsNoLongerNeeded[i].Date) < tonumber(GetServerTime() - (DaysToSave * SecsPerDay)) then
            table.remove(GFL_ItemsNoLongerNeeded, i)
         end
      end
      SendAddonMessage(ADDON_PREFIX, "A=S;", "GUILD")
      Utils.SetFramePosition(GFL_MainFrame, GFL_Options.MainFrame.xpos, GFL_Options.MainFrame.ypos, GFL_Main.MainFrame.ResetPosition)
   elseif event == "PLAYER_ENTERING_WORLD" then
      SendAddonMessage(ADDON_PREFIX, "A=S;", "GUILD")
   end
end

function GFL_Main.MainFrame.ResetPosition(self)
   self:SetPoint("CENTER", "UIParent")
end

function GFL_Main.RemoveItemByIndex(index, date)
   if index and type(index) == "number" then
      local item = GFL_ItemsNeeded[index].Item
      local t = {}
      t.Item = item
      if date then
         t.Date = date
      else
         t.Date = GetServerTime()
      end
      local delisted, delistedIndex = ItemKnownDelisted(item)
      if delisted then
         GFL_ItemsNoLongerNeeded[delistedIndex].Date = math.max(t.Date, GFL_ItemsNoLongerNeeded[delistedIndex].Date)
      else
         table.insert(GFL_ItemsNoLongerNeeded, 1, t)
      end
      table.remove(GFL_ItemsNeeded, index)
      if not date then
         AddonSendMessage(true, t.Item, t.Date)
      end
      GFL_Main.ScrollFrame.OnUpdate(GFL_MainFrame_ScrollFrame)
   end
end

function GFL_Main.OnLoad(self)
   GFL_MainFrame_OkButton:SetFrameLevel(8)

   success = RegisterAddonMessagePrefix(ADDON_PREFIX)
   Utils.RegisterEvents(self, GFL_Main.MainFrame.OnEvent, 'CHAT_MSG_ADDON', 'ADDON_LOADED', 'PLAYER_ENTERING_WORLD')
end

function GFL_Main.DraggingButton.OnLoad(self)
   self:RegisterForClicks("LeftButtonDown")
   self:RegisterForDrag("LeftButton")
end

function GFL_Main.DraggingButton.OnClick(self)
   Utils.GetCursorRelativePosition(GFL_MainFrame)
end

function GFL_Main.DraggingButton.OnDragStart(self)
   self:SetScript("OnUpdate", GFL_Main.DraggingButton.OnUpdate)
end

function GFL_Main.DraggingButton.OnDragStop(self)
   self:SetScript("OnUpdate", nil)
end

function GFL_Main.DraggingButton.OnUpdate(self)
   local x, y = Utils.GetFramePosition(GFL_MainFrame)
   Utils.SetFramePosition(GFL_MainFrame, x, y, GFL_Main.MainFrame.ResetPosition)
   GFL_Options.MainFrame.xpos = x
   GFL_Options.MainFrame.ypos = y
end

function GFL_Main.MainFrame.OnClickClose(self)
	PlaySound("igMainMenuQuit");
   Utils.ToggleFrame(GFL_MainFrame)
end

function GFL_Main.EditBox.OnClickOk(self)
   local editbox = GFL_MainFrame_EditBox
   local text = editbox:GetText()
   AddItem(text)
   editbox:SetText("")
end

function GFL_Main.EditBox.OnTextChanged(frame)
   local text = frame:GetText()
   if #text > 0 then
      GFL_MainFrame_OkButton:Show()
   else
      GFL_MainFrame_OkButton:Hide()
   end
end

function GFL_Main.EditBox.OnEnterPressed(self)
   GFL_Main.EditBox.OnClickOk(GFL_MainFrame_OkButton)
end

function GFL_Main.EditBox.OnEscapePressed(frame)
   frame:SetText("")
   frame:ClearFocus()
end

local function ScrollFrame_Update(self)
   local name= self:GetName()
   local scrollBar = _G[name.."ScrollBar"]
   local scrollUpButton = _G[ name.."ScrollBarScrollUpButton" ];
   local scrollDownButton = _G[ name.."ScrollBarScrollDownButton" ];

   if #GFL_ItemsNeeded > MaxScrollItemsShown then
      scrollBar:Show()
   else
      scrollBar:Hide()
      scrollUpButton:Disable();
      scrollDownButton:Disable();
   end

   if scrollBar:IsShown() then
      scrollBar:SetMinMaxValues(ScrollBarMin, ScrollBarMax)

      -- Arrow button handling
      if ( scrollBar:GetValue() == ScrollBarMin) then
         scrollUpButton:Disable();
      else
         scrollUpButton:Enable();
      end
      if ((scrollBar:GetValue()) == ScrollBarMax) then
         scrollDownButton:Disable();
      else
         scrollDownButton:Enable();
      end
   end
end

function GFL_Main.ScrollFrame.OnUpdate(self)
   local lineplusoffset=0
   ScrollFrame_Update(self)
   local offset = FauxScrollFrame_GetOffset(self)

   for line=1, MaxScrollItemsShown do
      lineplusoffset = line + ScrollBarOffset
      if lineplusoffset <= #GFL_ItemsNeeded then
         local item = Utils.GetItemLink(GFL_ItemsNeeded[lineplusoffset].Item)
         if item ~= GFL_ItemsNeeded[lineplusoffset].Item then
            GFL_ItemsNeeded[lineplusoffset].Item = item
         end
         _G["ScrollItem_"..line]:SetText(item)
         _G["ScrollItem_"..line]:Show()
         _G["RemoveButton_"..line]:Show()
         --_G["SignUpButton_"..line]:Show() --TODO: Add this functionality
      else
         _G["ScrollItem_"..line]:Hide()
         _G["RemoveButton_"..line]:Hide()
         --_G["SignUpButton_"..line]:Hide()  --TODO: Add this functionality
      end
   end
end

function GFL_Main.ScrollFrame.OnVerticalScroll(self, scrollPosition)
   if #GFL_ItemsNeeded > MaxScrollItemsShown then
      ScrollBarOffset = math.ceil((#GFL_ItemsNeeded - MaxScrollItemsShown) * (scrollPosition / (ScrollBarMax - ScrollBarMin)))
   else
      ScrollBarOffset = 0
   end
   GFL_Main.ScrollFrame.OnUpdate(self)
end

function GFL_Main.ScrollFrame.PopulateScrollFrame(self)
   local buffer = -10
   local index = 1
   repeat
      local i = index
      local item = self:CreateFontString("ScrollItem_"..i, "ARTWORK", "GameFontNormal")
      item:SetTextColor(1,1,1,1)
      item:SetText("Temp")
      if i == 1 then
         item:SetPoint("TOPLEFT", self, "TOPLEFT", math.abs(buffer), buffer)
      else
         item:SetPoint("TOPLEFT", _G["ScrollItem_"..(i-1)], "BOTTOMLEFT", 0, buffer)
      end
      item:Hide()

      if not MaxScrollItemsShown then
         MaxScrollItemsShown = math.floor(self:GetHeight() / (item:GetHeight() + math.abs(buffer)))
      end

      local removeButton = CreateFrame("Button", "RemoveButton_"..i, self, "UIPanelButtonTemplate")
      removeButton:SetText("X")
      removeButton:SetPoint("TOP", item, "TOP", 0, 5)
      removeButton:SetPoint("RIGHT", self, "RIGHT", -10, 0)
      removeButton:SetScript("OnClick", function () GFL_Main.RemoveItemByIndex(i) end)
      removeButton:Hide()

      local signUpButton = CreateFrame("Button", "SignUpButton_"..i, self, "UIPanelButtonTemplate")
      signUpButton:SetPoint("TOP", item, "TOP", 0, 5)
      signUpButton:SetPoint("RIGHT", removeButton, "LEFT", -10, 0)
      signUpButton:SetScript("OnClick", function () end)
      signUpButton:Hide()
      index = index + 1
   until index == (MaxScrollItemsShown + 1)
end

function GFL_Main.MiniMap.OnEvent(self, event, ...)
   local args = {...}
   if event == "ADDON_LOADED" and args[1] == ADDON_NAME then
      Utils.SetFramePosition(GFL_MMButton, GFL_Options.Minimap.xpos, GFL_Options.Minimap.ypos, GFL_Main.MiniMap.ResetPosition)
   end
end

function GFL_Main.MiniMap.OnLoad(self)
   self:RegisterForClicks("LeftButtonDown", "LeftButtonUp")
   self:RegisterForDrag("LeftButton")
   self:SetFrameLevel(8)
   Utils.RegisterEvents(self, GFL_Main.MiniMap.OnEvent, 'ADDON_LOADED')
end

function GFL_Main.MiniMap.OnClick(self, button, down)
   if down then
      GFL_UtilFuncs.GetCursorRelativePosition(self)
   else
      GFL_UtilFuncs.ToggleFrame(GFL_MainFrame);
   end
end

function GFL_Main.MiniMap.OnEnter(self)
   GFL_UtilFuncs.ShowToolTip(self)
end

function GFL_Main.MiniMap.OnLeave(self)
   GFL_UtilFuncs.HideToolTip(self)
end

function GFL_Main.MiniMap.OnDragStart(self, button)
   self:LockHighlight()
   self:SetScript("OnEnter", nil)
   self:SetScript("OnUpdate", GFL_Main.MiniMap.OnUpdate)
end

function GFL_Main.MiniMap.OnDragStop(self)
   self:UnlockHighlight()
   self:SetScript("OnUpdate", nil)
   self:SetScript("OnEnter", GFL_UtilFuncs.ShowToolTip)
end

function GFL_Main.MiniMap.OnUpdate(self)
   local x, y = GFL_UtilFuncs.GetFramePosition(self)
   GFL_UtilFuncs.SetFramePosition(self, x, y, GFL_Main.MiniMap.ResetPosition)
   GFL_Options.Minimap.xpos = x
   GFL_Options.Minimap.ypos = y
end

function GFL_Main.MiniMap.ResetPosition(self)
   self:SetPoint("CENTER", "Minimap", "CENTER", -64.9, -37.5) 
end
